// Changelog:
//   cleanups + port support (pl)
//
// Author: Olivier Valentin
// License: GPLv2

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <netdb.h>

#include <errno.h>

#include "nfspdctl.h"
#include <netinet/in.h>

static int getint() {
	int i;
	char c[10];
	do {
		fgets(c, 10, stdin);
	} while (sscanf(c, "%d", &i) != 1);
	return i;
}

/**
 * resolv : fills addr with parameters from iodline/wantedport
 * @addr: result
 * @iodline: line in the form <1.2.3.4|host.blah.xx>[:1234]
 *           if port not precised then it's 20000
 * @wantedport: if in the range 0..0xFFFF then it's used as port
 *              if no port is given in iodline
 */
static int resolv(struct sockaddr_in *addr, char *iodline, int wantedport) {
	char *iodname;
	char *port_str;
	int port;
	struct hostent *he;

	iodname = strdup(iodline);
	if (!iodname) {
		fprintf(stderr, "Error: strdup failed\n");
		return -1;
	}

	port_str = strchr(iodname, ':');
	if (port_str) {
		*port_str = 0;
		port_str++;
		port = atoi(port_str);
	} else {
		if (wantedport >= 0 && wantedport <= 0xFFFF) {
			port = wantedport;
		} else {
			port = 20000;
		}
	}

	if (port <0 || port > 0xFFFF) {
		fprintf(stderr, "Error: invalid port %d\n", port);
		return -1;
	}

	he = gethostbyname(iodname);
	if (he != NULL) {
		memcpy((char *)&(addr->sin_addr), he->h_addr, he->h_length);
	} else {
		fprintf(stderr, "Error: cannot resolve hostname '%s'\n", iodname);
		return -1;
	}
		
	addr->sin_family = AF_INET;
	addr->sin_port = htons(port);

	return 0;
}

static int faddiod() {
	struct sockaddr_in addr;
	char iodname[32];
	unsigned short port;

	printf("New iod\n");
	printf("IOD addr: ");
	if (!fgets(iodname, 32, stdin)) {
		fprintf(stderr, "\nError while reading from stdin\n");
		return -1;
	} else {
		char *lf;
		if ((lf = strchr(iodname, '\n'))) {
			*lf = '\0';
		}
	}

	printf("port: ");
	port = (unsigned short)getint();


	if (resolv(&addr, iodname, port)) {
		return -1;
	}
	
	return nfspd_addiod(&addr);
}

static int ffreeiod() {
	int ih;
	printf("Free Iod\n");
	printf("ih: ");
	ih = getint();
	return nfspd_freeiod(ih);
}

static int fbind() {
	int ih, iviod;
	printf("Bind Iod\n");
	printf("iviod: ");
	iviod = getint();
	printf("ih: ");
	ih = getint();
	return nfspd_bindiod(iviod, ih);
}

static int funbind() {
	int ih;
	printf("Unbind Iod\n");
	printf("ih: ");
	ih = getint();
	return nfspd_unbindiod(ih);
}

static int fsetnbviods() {
	int nb;
	printf("Set nbviods\n");
	printf("nbviods: ");
	nb = getint();
	return nfspd_setnbviods(nb);
}

static int fgetnbviods() {
	return nfspd_getnbviods();
}

static int fdrop() {
	return nfspd_drop();
}

static int fforward() {
	return nfspd_forward();
}

static int fnflush() {
	return nfspd_flush();
}

static int fgetviodsize() {
	int iviod;
	printf("iviod: ");
	iviod = getint();
	return nfspd_get_viod_size(iviod);
}

static void disp(int result) {
	printf("The result is: %d\n", result);
	if (result < 0) perror("syscall");
}

static int addbind(int argc, char **argv) {
	int it;
	struct sockaddr_in addr;
	int res;

	printf("flushing... ");
	res = nfspd_flush();
	if (res) {
		perror("flush:");
		return -1;
	}
	printf("ok\n");

	printf("Set nbviods to %d...\n", argc - 1);
	res = nfspd_setnbviods(argc - 1);
	if (res) {
		perror("setnbviods:");
		return -1;
	}
	printf("ok\n");

	for (it = 1; it < argc; it ++) {
		printf("Creating IOD: '%s'\n", argv[it]);
		if (resolv(&addr, argv[it], -1)) {
			continue;
		}
		res = nfspd_addiod(&addr);
		if (res < 0) {
			perror("addiod:");
			continue;
		}
		printf("  ih = %d\n", res);

		printf(" Binding to VIOD #%d\n", it - 1);
		res = nfspd_bindiod(it - 1, res);
		if (res) {
			perror("bind:");
			continue;
		}
	}
	
	printf("Forward...");
	res = nfspd_forward();
	if (res) {
		perror("forward:");
		return -1;
	}
	printf("ok\n");
	
	return 0;
}

int main(int argc, char **argv) {
	int quit = 0;

	if (argc > 1) {
		return addbind(argc, argv);
	}

	while (!quit) {
		printf("-------------------\n");
		printf("1  addiod\n");
		printf("2  freeiod\n");
		printf("3  bind\n");
		printf("4  unbind\n");
		printf("5  set_nbviods\n");
		printf("6  get_nbviods\n");
		printf("7  drop\n");
		printf("8  forward\n");
		printf("9  flush\n");
		printf("10 get_viod_size\n");
		printf("0 quit\n");
		switch (getint()) {
			case 1: disp(faddiod());
				break;
			case 2: disp(ffreeiod());
				break;
			case 3: disp(fbind());
				break;
			case 4: disp(funbind());
				break;
			case 5: disp(fsetnbviods());
				break;
			case 6: disp(fgetnbviods());
				break;
			case 7: disp(fdrop());
				break;
			case 8: disp(fforward());
				break;
			case 9: disp(fnflush());
				break;
			case 10: disp(fgetviodsize());
				break;
			case 0: quit = 1;
					  break;
			default:
		}
	}
	return 0;
}
